<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: PluginFactory.php 3226 2008-08-06 17:58:08Z bigdisk $
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/PluginPeer.php');
require_once('plugin/GFPlugin.php');

/**
 * This class builds GFPlugin objects. It is accessed as a Singleton.
 */
class PluginFactory {

	/**
	 * This variable will hold a list of the plugins that have been
	 * initialized. 
	 */
	private $initializedPlugins = null;
	
	/**
	 * This variable holds the global instance of the plugin factory. Only one instance
	 * is needed, we don't want to create the instances of the plugins more than once.
	 */
	private static $instance = null;

	private function __construct() {

	}
	
	/**
	 * Return the only (global) instance of the factory.
	 * @return PluginFactory
	 */
	public function getFactory() {
		if (is_null(self::$instance)) {
			self::$instance = new PluginFactory();
			self::$instance->initializePlugins();
		}
		
		return self::$instance;
	}

	/**
	 * @return	array	Associative array with all the available plugins
	 */
	public function getPlugins() {
		if (!is_array($this->initializedPlugins)) {
			$this->initializePlugins();
		}
		return $this->initializedPlugins;
	}
	
	/**
	 * Retrieve a GFPlugin by its name
	 * @param	string $name	Name of the plugin
	 * @return	GFPlugin
	 */
	public function getPluginByName($name) {
		$plugins = $this->getPlugins();
		if (!array_key_exists($name, $plugins)) {
			return null;
		}
		return $plugins[$name];
	}
	
	/**
	 * This methods makes sure that all the plugins are initialized properly.
	 */
	public function initializePlugins() {
		if (!is_array($this->initializedPlugins)) {
			$this->initializedPlugins = array();
			
			$c = new Criteria();
	//		$c->add(PluginPeer::PLUGIN_TYPE, GFPlugin::SITE_PLUGIN);
			$c->addAscendingOrderByColumn(PluginPeer::PLUGIN_ORDER);
			$plugins = PluginPeer::doSelect($c);
			
			$pluginsPath = GFConf::get('system.pluginsPath');
			
			foreach ($plugins as $plugin) {
				$name = $plugin->getPluginName();
				$classname = ucfirst($name).'Plugin'; 
				
				//include and instantiate each plugin
				$plugin_php = SystemUtils::buildFilePath($pluginsPath, '/'.$name.'/lib/'.$classname.'.php');
				if (!is_file($plugin_php)) {
					throw new Exception('Couldn\'t locate plugin file: '.$plugin_php);
				}

				// Plugin file was found... instantiate it
				require_once($plugin_php);
				$this->initializedPlugins[$name] = new $classname();

				//  Initialize each plugin
				$this->initializedPlugins[$name]->initialize();
			}
		}

		return;
	}
}

?>
